<h3>Contents</h3>

<ul>
   <li>&nbsp; &nbsp;<a href="{{ pathto('installation') }}"> Installation</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('overview') }}">     Overview</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('interactive') }}">   Interactive Displays</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('plotpanel') }}">    PlotPanel API</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('imagepanel') }}"> Image API</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('examples') }}">    Examples</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('other') }}"> Speciality Displays</a></li>
</ul>
<p>

<h3>WXMPLOT</h3>
<p>Current version: <b>{{ release }}</b></p>
<p>Install: &nbsp; <b>pip install wxmplot</b>
<p>Develop:
  &nbsp; <a href="https://github.com/newville/wxmplot/">github.com</a>
  <p>


<p>
